@@ -53,8 +53,8 @@ import (
 	installertypesovirt "github.com/openshift/installer/pkg/types/ovirt"
 	installertypesvsphere "github.com/openshift/installer/pkg/types/vsphere"
 
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
 	contributils "github.com/openshift/hive/contrib/pkg/utils"
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
 	"github.com/openshift/hive/pkg/constants"
 	"github.com/openshift/hive/pkg/gcpclient"
 	"github.com/openshift/hive/pkg/resource"
@@ -161,6 +161,8 @@ func NewInstallManagerCommand() *cobra.Command {
 	flags.StringVar(&im.LogLevel, "log-level", "info", "log level, one of: debug, info, warn, error, fatal, panic")
 	flags.StringVar(&im.WorkDir, "work-dir", "/output", "directory to use for all input and output")
 	flags.StringVar(&im.LogsDir, "logs-dir", "/logs", "directory to use for all installer logs")
+
+	cmd.AddCommand(NewInstallManagerAWSCredentials())
 	return cmd
 }
 
@@ -336,7 +338,7 @@ func (m *InstallManager) Run() error {
 
 	// Generate installer assets we need to modify or upload.
 	m.log.Info("generating assets")
-	if err := m.generateAssets(provision); err != nil {
+	if err := m.generateAssets(cd); err != nil {
 		m.log.Info("reading installer log")
 		installLog, readErr := m.readInstallerLog(provision, m, scrubInstallLog)
 		if readErr != nil {
@@ -526,14 +528,6 @@ func (m *InstallManager) copyFile(src, dst string) error {
 
 // cleanupFailedInstall allows recovering from an installation error and allows retries
 func (m *InstallManager) cleanupFailedInstall(cd *hivev1.ClusterDeployment, provision *hivev1.ClusterProvision) error {
-	if err := m.cleanupAdminKubeconfigSecret(); err != nil {
-		return err
-	}
-
-	if err := m.cleanupAdminPasswordSecret(); err != nil {
-		return err
-	}
-
 	infraID := provision.Spec.InfraID
 	if infraID == nil {
 		infraID = provision.Spec.PrevInfraID
@@ -547,6 +541,14 @@ func (m *InstallManager) cleanupFailedInstall(cd *hivev1.ClusterDeployment, prov
 		m.log.Warn("skipping cleanup as no infra ID set")
 	}
 
+	if err := m.cleanupAdminKubeconfigSecret(); err != nil {
+		return err
+	}
+
+	if err := m.cleanupAdminPasswordSecret(); err != nil {
+		return err
+	}
+
 	return nil
 }
 
@@ -665,7 +667,7 @@ func cleanupFailedProvision(dynClient client.Client, cd *hivev1.ClusterDeploymen
 
 // generateAssets runs openshift-install commands to generate on-disk assets we need to
 // upload or modify prior to provisioning resources in the cloud.
-func (m *InstallManager) generateAssets(provision *hivev1.ClusterProvision) error {
+func (m *InstallManager) generateAssets(cd *hivev1.ClusterDeployment) error {
 	m.log.Info("running openshift-install create manifests")
 	err := m.runOpenShiftInstallCommand("create", "manifests")
 	if err != nil {
@@ -673,6 +675,23 @@ func (m *InstallManager) generateAssets(provision *hivev1.ClusterProvision) erro
 		return err
 	}
 
+	// Configure for install of an STS cluster if specified:
+	boundSASigningKey := os.Getenv(constants.BoundServiceAccountSigningKeyEnvVar)
+	if boundSASigningKey != "" {
+		tlsDir := filepath.Join(m.WorkDir, "tls")
+		dest := filepath.Join(tlsDir, constants.BoundServiceAccountSigningKeyFile)
+		m.log.WithField("src", boundSASigningKey).WithField("dest", dest).
+			Info("copying bound service account signing key")
+		if err := os.MkdirAll(tlsDir, 0700); err != nil {
+			m.log.WithError(err).Errorf("error creating %s directory", tlsDir)
+			return err
+		}
+		log.WithField("dest", dest).Info("copying bound SA signing key")
+		if err := m.copyFile(boundSASigningKey, dest); err != nil {
+			return err
+		}
+	}
+
 	if src := m.ManifestsMountPath; isDirNonEmpty(src) {
 		m.log.Info("copying user-provided manifests")
 		dest := filepath.Join(m.WorkDir, "manifests")
@@ -911,6 +930,12 @@ func (m *InstallManager) gatherClusterLogs(cd *hivev1.ClusterDeployment) error {
 	cmd.Env = append(cmd.Env, fmt.Sprintf("KUBECONFIG=%s", filepath.Join(m.WorkDir, "auth", "kubeconfig")))
 	stdout, err := cmd.Output()
 	m.log.Infof("must-gather output: %s", stdout)
+	if err != nil {
+		return err
+	}
+	// Creating a compressed file of the must-gather directory in m.LogsDir
+	tarCmd := exec.Command("tar", "cvaf", filepath.Join(m.LogsDir, "must-gather.tar.gz"), destDir)
+	err = tarCmd.Run()
 	return err
 }
 
@@ -1208,7 +1233,7 @@ func uploadAdminPassword(provision *hivev1.ClusterProvision, m *InstallManager)
 	return s, nil
 }
 
-func createWithRetries(obj runtime.Object, m *InstallManager) error {
+func createWithRetries(obj client.Object, m *InstallManager) error {
 	logger := m.log.WithField("kind", obj.GetObjectKind().GroupVersionKind().Kind)
 
 	metaObj, err := meta.Accessor(obj)
@@ -1284,7 +1309,7 @@ func waitForProvisioningStage(provision *hivev1.ClusterProvision, m *InstallMana
 	if err != nil {
 		return errors.Wrap(err, "could not get the GVK for clusterprovisions")
 	}
-	restClient, err := apiutil.RESTClientForGVK(gvk, config, scheme.Codecs)
+	restClient, err := apiutil.RESTClientForGVK(gvk, false, config, scheme.Codecs)
 	if err != nil {
 		return errors.Wrap(err, "could not create REST client")
 	}
